/*
 *  Copyright (C) 2021 Texas Instruments Incorporated
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* This example demonstrates the implementation of SHA */

#include <string.h>
#include <kernel/dpl/DebugP.h>
#include "ti_drivers_config.h"
#include "ti_drivers_open_close.h"
#include "ti_board_open_close.h"

/* SHA512 length */
#define APP_SHA512_LENGTH               (64U)
/* SHA256 length */
#define APP_SHA256_LENGTH               (32U)
/* Alignment */
#define APP_SHA_ALIGNMENT_FOR_BUF       (128U)
/*Input and output buf length*/
#define APP_SHA_IN_OUT_BUF_LENGTH       (2048U)

/*Inout test buffer for sha computation */
static uint8_t* gCryptoShaTestInputBuf = (uint8_t*)0x60200000;
/*Output test buffer for sha computation */
uint8_t gCryptoShaTestOutputBuf[APP_SHA_IN_OUT_BUF_LENGTH] __attribute__ ((aligned (APP_SHA_ALIGNMENT_FOR_BUF)));

/* SHA-512 test vectors,this is expected hash for the test buffer */
static uint8_t gCryptoSha512TestSum[APP_SHA512_LENGTH] = {
  0x4a, 0x22, 0xec, 0xd9, 0xae, 0xe2, 0x35, 0xdf, 0x6a, 0x2f, 0xba, 0x5d,
  0xc1, 0x99, 0xfa, 0x83, 0x85, 0xf7, 0x3a, 0x62, 0xb8, 0x84, 0x7a, 0x60,
  0x48, 0x90, 0x28, 0xdb, 0x6e, 0xb6, 0x82, 0x88, 0x5d, 0x01, 0x9f, 0xd4,
  0xa6, 0xe2, 0x05, 0x53, 0x10, 0x6b, 0x72, 0xd3, 0xf2, 0xab, 0xa1, 0xa4,
  0xaa, 0xf3, 0x32, 0x54, 0x7f, 0x05, 0x61, 0x8d, 0x66, 0x25, 0x70, 0x6a,
  0xd1, 0x3f, 0x1d, 0xf8
};

/* Context memory */
static Crypto_Context gCryptoShaContext __attribute__ ((aligned (SA2UL_CACHELINE_ALIGNMENT)));
/* Context Object */
SA2UL_ContextObject  gSa2ulCtxObj __attribute__ ((aligned (SA2UL_CACHELINE_ALIGNMENT)));

void crypto_sha(void *args)
{
    int32_t             status;
    Crypto_Handle       shaHandle;
    SA2UL_ContextParams ctxParams;

    Drivers_open();
    Board_driversOpen();

    DebugP_log("[CRYPTO] SHA example started ...\r\n");

    uint32_t n_tests = 25000, cnt = 0;

    for(uint32_t i = 0; i < n_tests; i++) {
        shaHandle = Crypto_open(&gCryptoShaContext);
        DebugP_assert(shaHandle != NULL);

        /* Configure secure context */
        ctxParams.opType    = SA2UL_OP_AUTH;
        ctxParams.hashAlg   = SA2UL_HASH_ALG_SHA2_512;
        ctxParams.inputLen  = APP_SHA_IN_OUT_BUF_LENGTH;
        gSa2ulCtxObj.totalLengthInBytes = APP_SHA_IN_OUT_BUF_LENGTH;

        status = SA2UL_contextAlloc(gCryptoShaContext.drvHandle, &gSa2ulCtxObj, &ctxParams);
        DebugP_assert(SystemP_SUCCESS == status);

        /* Perform SHA operation */
        status = SA2UL_contextProcess(&gSa2ulCtxObj,gCryptoShaTestInputBuf, APP_SHA_IN_OUT_BUF_LENGTH, gCryptoShaTestOutputBuf);
        DebugP_assert(SystemP_SUCCESS == status);

        /*Function to free secure context configuration*/
        status = SA2UL_contextFree(&gSa2ulCtxObj);
        DebugP_assert(SystemP_SUCCESS == status);

        /*comparing result with expected test results*/
        if(memcmp(gSa2ulCtxObj.computedHash, gCryptoSha512TestSum, APP_SHA512_LENGTH) != 0)
        {
            DebugP_log("#%d: FAIL\r\n", i);
        }
        else
        {
            cnt++;
            DebugP_log("#%d: PASS\r\n", i);
        }

        memset(&gSa2ulCtxObj, 0x0, sizeof(gSa2ulCtxObj));

        /* Close SHA instance */
        status = Crypto_close(shaHandle);
        DebugP_assert(SystemP_SUCCESS == status);
    }

    DebugP_log("#PASS: %d, #FAIL: %d\r\n", cnt, n_tests - cnt);

    Board_driversClose();
    Drivers_close();

    return;
}
